#!/bin/sh
#
# Cross-correlation of noise data.
# Script finds all possible combinations.
# See 1_README_example_pcc5h for further informations
#
# Last change: 24/07/2019, schimmel@ictja.csic.es
#--------------------------------------

# Executables:
#-------------
pcc="../../pcc5iMP"
norm="../../norm_tavg_white"

# Frequency band pass:
#---------------------
bp1=1.0
bp2=2.0

# Working dir:
#-------------
dir0=`pwd `

# Loop over all data:
#--------------------
for i in 2009.???.??
do
dir=$dir0"/"$i

### Alternative strategy to define loop:
###for i in `seq -w 1 366`
###do
###dir=$dir0"/2009."$i".00"

# if data dir exists then goto data dir: 
#---------------------------------------
if test -d $dir
then
cd $dir
echo $dir
nl=` ls -1 *SAC | wc -l | tail -1 | awk '{ if ($1 > 1) print "2" }' `
else
nl=1
fi

#process if nl=2:
#----------------
## alternative in case problems with if:
##  if [ "$nl" -eq "2" ]
##  if [ "$nl" == "2" ]
if [ "$nl" = "2" ]
then

# Pre-processing:
#----------------
# BP:
sac << END
r *HZ*SAC 
rmean
rtrend
bp co $bp1 $bp2 p 2
w append bp
quit
END

# running absolute mean normalization + whitening:
# (only for the classical cross-correlation)
#-------------------------------------------------
for f in *HZ*SACbp
do
$norm $f N=0 white
done

# Bandpass whitened data:
#------------------------
# BP:
sac << END
r *HZ*SACbpn
rmean
rtrend
bp co $bp1 $bp2 p 2
w over
quit
END

# Correlation loops:
#-------------------
n1=`ls -1 *HZ*bpn | wc -l | awk '{ print $1}'`
# start n1 while-loop 
while [ "$n1" -ge "1" ]
do

f1=` ls -1 *HZ*bpn | tail -$n1 | head -1 ` 
f3=` echo $f1 | sed 's/bpn/bp/' `
n2=` echo $n1 | awk '{ print $1-1}' ` 

#start n2 while loop:
while [ "$n2" -ge "1" ]
do

f2=` ls -1 *HZ*bpn | tail -$n2 | head -1 ` 
f4=` echo $f2 | sed 's/bpn/bp/' `

cp $f1 d1.sac
cp $f2 d2.sac
cp $f3 d3.sac
cp $f4 d4.sac

sac <<- %
r d1.sac d2.sac
sync
w over
setbb a &1,b
setbb b &2,b
setbb c &1,e
setbb d &2,e
cut ( (max %a %b%) + 1 ) ( (min %c %d%) - 1 )
r d1.sac d2.sac
write over
cut off
r d3.sac d4.sac
sync
w over
setbb a &1,b
setbb b &2,b
setbb c &1,e
setbb d &2,e
cut ( (max %a %b%) + 1 ) ( (min %c %d%) - 1 )
r d3.sac d4.sac
write over
quit
%

sta1=`echo $f1 | awk '{ print substr($1,16,4)}' `
sta2=`echo $f2 | awk '{ print substr($1,16,4)}' `
dat=`echo $f1 | awk '{ print substr($1,1,15)}' `
paar="$sta1"Z_"$sta2"Z
paar0=../"$paar"
mkdir -p $paar0

# cross-correlate time freq - normalized data (CCGN):
# (this can also be computed with sac!)
#========================================================
par=" isac osac tl1=-50 tl2=50 nn nsmin=4000"
echo CCGN: $paar0"/"$dat"ccgnn_"$paar $par
$pcc d1.sac d2.sac ccgn=$paar0"/"$dat"ccgnn_"$paar $par

# cross-correlate data without time freq - normalization (PCC):
#===============================================================
par=" isac osac tl1=-50 tl2=50 nn pov=2 nsmin=4000"
echo PCC: $paar0"/"$dat"pcc2_"$paar $par
$pcc d3.sac d4.sac pcc=$paar0"/"$dat"pcc2_"$paar $par
\rm -f d?.sac 

n2=`echo $n2 | awk '{ print $1-1}' ` 
#end n2 while-loop
done

n1=`echo $n1 | awk '{ print $1-1}' `
#end n1 while-loop
done 

# clean up 
\rm -f *SACbp*
fi
# end data for-loop:
done

exit
