#!/bin/sh
#
# This is like do_pcc.cmd, but compute also the classical
# cross-correlation (with and without normalizations).

# Script finds all possible combinations. See do_pcc.cmd 
# and 1_README_example_pcc for further information.
#
# 25/07/2019, schimmel@ictja.csic.es
#--------------------------------------

# Executables:
#-------------
pcc="../../pcc5iMP"
norm="../../norm_tavg_white"

# Frequency band pass:
#---------------------
bp1=1.0 
bp2=3.0

# Working dir:
#-------------
dir0=`pwd`

# Loop over all data:
#--------------------
for i in 2018.???
do
dir=$dir0"/"$i
cd $dir

# Pre-processing:
#----------------
# BP:
sac << END
r 20*HZ.sac
rmean
rtrend
bp co $bp1 $bp2 p 2
w append bp
quit
END

# running 1-bit + whitening:
# (only used for the classical cross-correlation)
#-------------------------------------------------
for f in *HZ*sacbp
do
$norm $f N=0 white
done

# Bandpass whitened data:
#------------------------
# BP:
sac << END
r *HZ*sacbpn
rmean
rtrend
bp co $bp1 $bp2 p 2
w over
quit
END

ls -1 20*HZ.sac | awk '{print substr($1,1,15)}' | uniq > tmpo

# While-loop over different hours
#--------------------------------
while read line;
do

# Correlation loops:
#-------------------
n1=`ls -1 $line*HZ*bpn | wc -l | awk '{ print $1}'`
# start n1 while-loop 
while [ "$n1" -ge "1" ]
do

f1=` ls -1 $line*HZ*bpn | tail -$n1 | head -1 ` 
f3=` echo $f1 | sed 's/bpn/bp/' `
n2=` echo $n1 | awk '{ print $1-1}' ` 

#start n2 while-loop:
while [ "$n2" -ge "1" ]
do

f2=` ls -1 $line*HZ*bpn | tail -$n2 | head -1 ` 
f4=` echo $f2 | sed 's/bpn/bp/' `

sac <<- %
r $f1 $f2
sync
w d1.sac d2.sac
rh d1.sac d2.sac
setbb a &1,b
setbb b &2,b
setbb c &1,e
setbb d &2,e
cut ( (max %a %b%) + 1 ) ( (min %c %d%) - 1 )
r d1.sac d2.sac
write over
cut off
r $f3 $f4
sync
w  d3.sac d4.sac
rh  d3.sac d4.sac
setbb a &1,b
setbb b &2,b
setbb c &1,e
setbb d &2,e
cut ( (max %a %b%) + 1 ) ( (min %c %d%) - 1 )
r  d3.sac d4.sac
write over
quit
%

sta1=`echo $f1 | awk '{ print substr($1,20,4)}' `
sta2=`echo $f2 | awk '{ print substr($1,20,4)}' `
dat=`echo $f1 | awk '{ print substr($1,1,15)}' `
paar="$sta1"Z_"$sta2"Z
paar0=../"$paar"
mkdir -p $paar0

# cross-correlate time freq - normalized data (CCGN):
# (this can also be computed with sac!)
#========================================================
echo CCGN: $paar0"/"$dat"ccgn_"$paar".sac" $par
par=" isac osac tl1=-30 tl2=30 nn nsmin=45000 "
$pcc d1.sac d2.sac $par ccgn=$paar0"/"$dat"ccgnn_"$paar
\rm -f d1.sac d2.sac

# cross-correlate data without time freq - normalization (PCC):
#===============================================================
echo XXX=PCC,CCGN: $paar0"/"$dat"XXX_"$paar".sac" $par
$pcc d3.sac d4.sac $par pov=2 pcc=$paar0"/"$dat"pcc2_"$paar ccgn=$paar0"/"$dat"ccgn_"$paar

n2=`echo $n2 | awk '{ print $1-1}' ` 
#end n2 while-loop
done

n1=`echo $n1 | awk '{ print $1-1}' `
#end n1 while-loop
done 

# End while-loop over hours
done < tmpo

# clean up 
\rm -f *sacbp*
\rm -f d?.sac tmpo

# end data for-loop over i:
done

exit
